@@ -190,7 +190,7 @@ class Database:
         database_str = 'database=' + 'hk_earning_calendar_by_code'
         code_str = 'code=' + code
         link_str = link_url + code_str + '&' + token_str + '&' + database_str
-        
+        print(link_str)
         response = requests.get(link_str)
         result = ast.literal_eval(json.loads(response.content))
 
@@ -666,13 +666,16 @@ class Database:
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             
-            if deri == 'opt':            
+            if deri == 'opt':
+                df = df.rename(columns={'contract_month':'year_month'})
+                df['year_month'] = df['year_month'].str.replace('-','')
                 df['year_month'] = pd.to_datetime(df['year_month'], format='%b%y')
                 df = df.sort_values(by=['date','year_month'])
                 df['year_month'] = df['year_month'].dt.strftime('%b-%y')
-                cols = list(df.columns)
-                cols = [cols[-1]] + cols[:-1]
-                df = df[cols]
+                if code.isdigit() == True:
+                    cols = list(df.columns)
+                    cols = [cols[-1]] + cols[:-1]
+                    df = df[cols]
 
             elif deri == 'fut':
                 df = df.sort_values(by='date')
@@ -738,7 +741,10 @@ class Database:
         try:
             day_diff = (datetime.datetime.strptime(str(end_date),'%Y%m%d').date() - \
                                    datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
+        except:
+            exceed_max_day_diff = False
 
+            '''
             if freq == '1T':
                 exceed_max_day_diff = day_diff <= 36
             elif freq == '5T':
@@ -751,8 +757,8 @@ class Database:
                 exceed_max_day_diff = day_diff <= 1080
             elif freq == '1D':
                 exceed_max_day_diff = day_diff <= 2160
-        except:
-            exceed_max_day_diff = False
+            '''
+        exceed_max_day_diff = True
         
         check_bool_dict = {'freq_valid': freq_valid,
                            'code_isdigit': code_isdigit,
@@ -806,14 +812,17 @@ class Database:
                 str(start_date), '%Y%m%d').date()
         except:
             end_after_start_date = False
-            
+        
+        '''
         try:
             day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
                         datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
             exceed_max_day_diff = day_diff <= 36
         except:
             exceed_max_day_diff = False
-
+        '''
+        exceed_max_day_diff = True
+        
         check_bool_dict = {'code_length': code_length,
                            'start_date_length': start_date_length,
                            'start_date_is_int': start_date_is_int,
@@ -907,12 +916,15 @@ class Database:
         except:
             end_after_start_date = False
         
+        '''
         if max_day < 9999:
             day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
                         datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
             exceed_max_day_diff = day_diff <= max_day
         else:
             exceed_max_day_diff = True
+        '''
+        exceed_max_day_diff = True
             
         check_bool_dict = {
             'start_date_length': start_date_length,
@@ -961,13 +973,15 @@ class Database:
         except:
             end_after_start_date = False
         
+        '''
         try:
             day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
                         datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
             exceed_max_day_diff = day_diff <= 360
         except:
             exceed_max_day_diff = False
-
+        '''
+        exceed_max_day_diff = True
         
         check_bool_dict = {'ccass_id_len' : ccass_id_len,
                            'start_date_length': start_date_length,
@@ -1022,14 +1036,17 @@ class Database:
                 str(start_date), '%Y%m%d').date()
         except:
             end_after_start_date = False
-
+        
+        '''
         try:
             day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
                         datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
             exceed_max_day_diff = day_diff <= 2160
         except:
             exceed_max_day_diff = False
-
+        '''
+        exceed_max_day_diff = True
+        
         check_bool_dict = {
             'code_isdigit': code_isdigit,
             'code_length': code_length,
@@ -1086,14 +1103,16 @@ class Database:
                 str(start_date), '%Y%m%d').date()
         except:
             end_after_start_date = False
-
+        
+        '''
         try:
             day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
                         datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
             exceed_max_day_diff = day_diff <= 360
         except:
             exceed_max_day_diff = False
-
+        '''
+        exceed_max_day_diff = True
 
         check_bool_dict = {
             'code_isdigit': code_isdigit,
@@ -1161,7 +1180,8 @@ class Database:
                 str(start_date), '%Y%m%d').date()
         except:
             end_after_start_date = False
-
+        
+        '''
         try:
             day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
                         datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
@@ -1169,7 +1189,9 @@ class Database:
                                   (deri == 'fut' and day_diff <= 2160)
         except:
             exceed_max_day_diff = False
-
+        '''
+        exceed_max_day_diff = True
+        
         check_bool_dict = {'code_length': code_length,
                            'start_date_length': start_date_length,
                            'start_date_is_int': start_date_is_int,
@@ -1183,4 +1205,95 @@ class Database:
                            'exp_type': exp_type}
 
         return check_bool_dict
+    
+    
+    ################################ other tools ################################ 
 
+    def get_holiday_from_gov(self, year):
+        r = requests.get('https://www.gov.hk/en/about/abouthk/holiday/' + year + '.htm')
+        soup = BeautifulSoup(r.content.decode('utf-8'), 'lxml')
+        items = soup.find('table').find_all('tr')
+
+        holiday_date_list = []
+        half_day_mkt_date_list = []
+
+        for item in items[1:]:
+            tds = item.find_all('td')
+            holiday_date = tds[1].text + ' ' + year
+            holiday_date = datetime.datetime.strptime(holiday_date, '%d %B %Y').date()
+            holiday_name = tds[0].text.lower()
+            holiday_date_list.append(holiday_date)
+            if 'lunar new year' in holiday_name and 'the' not in holiday_name:
+                lin30_date = holiday_date - datetime.timedelta(days=1)
+                half_day_mkt_date_list.append(lin30_date)
+
+        xmax_eva_date = datetime.date(int(year), 12, 24)
+        if xmax_eva_date.weekday() < 5:
+            half_day_mkt_date_list.append(xmax_eva_date)
+
+        xmax_eva_date = datetime.date(int(year), 12, 24)
+        if xmax_eva_date.weekday() < 5:
+            half_day_mkt_date_list.append(xmax_eva_date)
+        year_eva_date = datetime.date(int(year), 12, 31)
+        if year_eva_date.weekday() < 5:
+            half_day_mkt_date_list.append(year_eva_date)
+
+        return holiday_date_list, half_day_mkt_date_list
+
+    def get_hk_holiday_and_expiry_date(self, start_year, end_year, format='int'):
+        holiday_date_list = []
+        half_day_mkt_date_list = []
+
+        for i in range(end_year - start_year + 1):
+            this_year = end_year - i
+            this_year_str = str(this_year)
+            new_holiday_date_list, new_half_day_mkt_date_list = self.get_holiday_from_gov(this_year_str)
+            holiday_date_list = holiday_date_list + new_holiday_date_list
+            half_day_mkt_date_list = half_day_mkt_date_list + new_half_day_mkt_date_list
+
+        start_date = datetime.date(start_year, 1, 1)
+        end_date = datetime.date(end_year, 12, 31)
+        date_diff = (end_date - start_date).days
+
+        expiry_date_list = []
+        for i in range(date_diff):
+            date = end_date - datetime.timedelta(days=i)
+            last_date = date + datetime.timedelta(days=1)
+            if last_date.day == 1:
+                trading_days = 0
+                for j in range(7):
+                    test_date = date - datetime.timedelta(days=j)
+                    if test_date.weekday() < 5 and test_date not in holiday_date_list:
+                        trading_days += 1
+                    if trading_days == 2:
+                        if format == 'str':
+                            expiry_date = test_date.strftime('%Y%m%d')
+                        elif format == 'int':
+                            expiry_date = int(test_date.strftime('%Y%m%d'))
+                        elif format == 'datetime':
+                            expiry_date = test_date
+                        expiry_date_list.append(expiry_date)
+                        break
+
+        holiday_list = []
+        for day in holiday_date_list:
+            if format == 'str':
+                holiday = day.strftime('%Y%m%d')
+            elif format == 'int':
+                holiday = int(day.strftime('%Y%m%d'))
+            elif format == 'datetime':
+                holiday = day
+            holiday_list.append(holiday)
+
+        half_day_mkt_list = []
+        for day in half_day_mkt_date_list:
+            if format == 'str':
+                holiday = day.strftime('%Y%m%d')
+            elif format == 'int':
+                holiday = int(day.strftime('%Y%m%d'))
+            elif format == 'datetime':
+                holiday = day
+            half_day_mkt_list.append(holiday)
+
+        dict1 = {'expiry_date': expiry_date_list, 'public_holiday': holiday_list, 'half_day_mkt': half_day_mkt_list}
+        return dict1
